// ================================================
// AreaSelector
// ================================================
function AreaSelector() {
	LOG__("Object: AreaSelector");
	this.area = new ForecastArea();
	this.createPrefSelector();
}

AreaSelector.prototype.onChangeForm = function(theForm) {
	LOG__("AreaSelector.onChangeForm(theForm)");
	var pref    = theForm.prefSelect.options.selectedIndex;
	var section = theForm.sectionSelect.options.selectedIndex;
	var city    = theForm.citySelect.options.selectedIndex;

	// If new pref is selected, set section number to 0
	if (pref != this.area.pref) {
		section = 0;
		city = 0;
	} else {
		if (section != this.area.section)
			city = 0;
	}

	var newArea = new ForecastArea(pref, section, city);
	this.setArea(newArea);
}

AreaSelector.prototype.setArea = function(theArea) {
	LOG__("AreaSelector.setArea(theArea)");
	this.area.setArea(theArea);
	this.rewriteSelectors();
}

AreaSelector.prototype.rewriteSelectors = function() {
	LOG__("AreaSelector.rewriteSelectors()");
	this.createSectionSelector();
	this.createCitySelector();
	this.selectCurrentIndex();
}

AreaSelector.prototype.createPrefSelector = function() {
	LOG__("AreaSelector.createPrefSelector()");
	var select = document.getElementById("prefSelect");
	var option;
	var i;

	// append OPTION elements
	for (i = 0; i < areaDictionary.prefs.length; i++) {
		option = document.createElement("option");
		option.setAttribute("value", i);
		option.innerText = getLocalizedString(areaDictionary.prefs[i].name);
		select.appendChild(option);
	}
}

AreaSelector.prototype.createSectionSelector = function() {
	LOG__("AreaSelector.createSectionSelector()");
	var select = document.getElementById("sectionSelect");
	var i;
	var option;
	var child;
	var sampleArea = new ForecastArea();
	var newArea;

	var numOfSections = areaDictionary.prefs[this.area.pref].array.length;

	// remove OPTION elements
	while (select.hasChildNodes())
		select.removeChild(select.lastChild);

	// append OPTION elements
	for (i = 0; i < numOfSections; i++) {

		option = document.createElement("option");
		option.setAttribute("value", i);
		option.innerText = getLocalizedString(
			areaDictionary.prefs[this.area.pref].array[i].name);
		select.appendChild(option);
	}
}

AreaSelector.prototype.createCitySelector = function() {
	LOG__("AreaSelector.createCitySelector()");
	var select = document.getElementById("citySelect");
	var i;
	var option;
	var child;
	var sampleArea = new ForecastArea();

	// remove OPTION elements
	while (select.hasChildNodes())
		select.removeChild(select.lastChild);

	// append OPTION elements
	for (i = 0; i < areaDictionary.prefs[this.area.pref].array[this.area.section].city.length; i++) {
		sampleArea.setAreaWithCode(this.area.pref, this.area.section, i);
		option = document.createElement("option");
		option.setAttribute("value", i);
		option.innerText = getLocalizedString(sampleArea.stringCity());
		select.appendChild(option);
	}
}

AreaSelector.prototype.selectCurrentIndex = function() {
	LOG__("AreaSelector.selectCurrentIndex()");
	document.getElementById("prefSelect").selectedIndex    = this.area.pref;
	document.getElementById("sectionSelect").selectedIndex = this.area.section;
	document.getElementById("citySelect").selectedIndex    = this.area.city;
}

// ================================================
// Presets
// ================================================
function Presets() {
	LOG__("Object: Presets");
	var i;

	this.area = new Array();
	this.defaultArea = new Array();

	for (i = 0; i < 4; i++) {
		this.area.push(new ForecastArea());
		this.defaultArea.push(new ForecastArea());
	}

	this.defaultArea[0].setAreaWithCode(18, 0, 0); // Tokyo
	this.defaultArea[1].setAreaWithCode(33, 0, 0); // Osaka
	this.defaultArea[2].setAreaWithCode(24, 0, 0); // Aichi
	this.defaultArea[3].setAreaWithCode(38, 0, 0); // Okayama
	
	this.loadAllArea();
}

Presets.prototype.loadAllArea = function() {
	LOG__("Object: Presets.loadAllArea");
	var i;

	for (i = 0; i < 4; i++)
		this.loadArea(i);
}

Presets.prototype.loadArea = function(index) {
	LOG__("Presets.loadArea(" + index + ")");
	if (!window.widget) return false;

	var pref, section, city;
	var dirtyFlag = false;

	// Load pref
	pref = widget.preferenceForKey("preset" + index + "pref");
	pref = Math.floor(pref);
	if (isNaN(pref) || (pref < 0) || (areaDictionary.prefs.length <= pref)) {
		pref = this.defaultArea[index].pref;
		dirtyFlag = true;
	}

	// Load section
	section = widget.preferenceForKey("preset" + index + "section");
	section = Math.floor(section);
	if (isNaN(section) || (section < 0)
		|| (areaDictionary.prefs[pref].array.length <= section)) {
		section = this.defaultArea[index].section;
		dirtyFlag = true;
	}

	// Load city
	city = widget.preferenceForKey("preset" + index + "city");
	city = Math.floor(city);
	if (isNaN(city) || (city < 0)
		|| (areaDictionary.prefs[pref].array[section].city.length <= city)) {
		city = this.defaultArea[index].city;
		dirtyFlag = true;
	}

	this.area[index].setAreaWithCode(pref, section, city);
	this.rewrite(index);
	if (dirtyFlag)
		this.saveArea(index);
}

Presets.prototype.setArea = function(index, newArea) {
	LOG__("Presets.setArea(" + index + ", newArea)");
	var dirtyFlag = !this.area[index].isEqualTo(newArea);

	if (dirtyFlag) {
		this.area[index].setArea(newArea);
		this.rewrite(index);
		this.saveArea(index);
	}
}

Presets.prototype.rewrite = function(index) {
	LOG__("Presets.rewrite(" + index + ")");

	var targetID = "preset" + index;
	var children;
	var locationString;
	var i;

	// Generate the string
	locationString =
		getLocalizedString(this.area[index].stringPref()) + " " + 
		getLocalizedString(this.area[index].stringSection()) + "-" + 
		getLocalizedString(this.area[index].stringCity()); 
	
	children = document.getElementById(targetID).childNodes;

	// Set innerText
	for (i = 0; i < children.length; i++) {
		if ((children[i].tagName == "SPAN") && 
				(children[i].getAttribute("class") == "location") ) {
			children[i].innerText = locationString;
		}
	}
}

Presets.prototype.saveArea = function(index) {
	LOG__("Presets.saveArea(" + index + ")");
	if (!window.widget) return false;

	widget.setPreferenceForKey(this.area[index].pref, "preset" + index + "pref");
	widget.setPreferenceForKey(this.area[index].section, "preset" + index + "section");
	widget.setPreferenceForKey(this.area[index].city, "preset" + index + "city");
}

